package com.jty.wsxt.application.service.impl;

import com.jty.wsxt.application.service.AuthRoleServer;
import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.auth.authority.AuthAuthority;
import com.jty.wsxt.domain.model.auth.role.AuthRole;
import com.jty.nrsc.infrastructure.support.BusinessException;
import com.jty.nrsc.infrastructure.support.ResultCode;
import com.jty.nrsc.interfaces.dto.AuthRoleDto;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

/**
 * AuthRoleServerImpl
 *
 * @author Manjiajie
 * @since 2019-6-13 11:03:18
 */
@Service
public class AuthRoleServerImpl implements AuthRoleServer {

    private static int admin_role_id = 1;
    @Override
    public Page<AuthRole> findPageRolesByPlatform(String platform, AuthRoleDto roleDto) {
        AuthRole authRole = new AuthRole();
        authRole.setPlatform(platform);
        return  DomainRegistry.roleRepository().findAll( Example.of(authRole),roleDto.getPageable());
    }

    @Override
    public void addRole(AuthRoleDto roleDto, String platform) {
        AuthRole authRole = DomainRegistry.roleRepository().findByNameAndPlatform(roleDto.getName(),platform).orElse(null);
        if(authRole != null){
            throw new BusinessException(ResultCode.ROLE_HAS_EXIST);
        }
        authRole = new AuthRole();
        authRole.setName(roleDto.getName());
        authRole.setPlatform(platform);
        authRole.save();
    }

    @Override
    public void updateRole(AuthRoleDto roleDto, String platform) {
        AuthRole authRoleById = DomainRegistry.roleRepository().findById(roleDto.getId()).orElseThrow(()->new BusinessException(ResultCode.DATA_IS_WRONG));
       if(roleDto.getId() == 1){
           if(CollectionUtils.isEmpty(roleDto.getAuthAuthoritiesIds())){
             throw new BusinessException(ResultCode.ADMIN_AUTH_NULL);
           }
       }

        AuthAuthority.cleanAuthority(authRoleById);

        if(CollectionUtils.isNotEmpty(roleDto.getAuthAuthoritiesIds())){
            authRoleById.addAuthoritiesByIds(roleDto.getAuthAuthoritiesIds());
        }
        authRoleById.save();
    }

    @Override
    public AuthRole getRoleById(Integer roleId) {
        return DomainRegistry.roleRepository().findById(roleId).orElseThrow(()->new BusinessException(ResultCode.DATA_IS_WRONG));
    }
}
