package com.jty.wsxt.application.service.impl;

import com.jty.wsxt.application.service.AuthUserServer;
import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.auth.history.*;
import com.jty.wsxt.domain.model.auth.user.AuthUser;
import com.jty.nrsc.infrastructure.DomainEventPublisher;
import com.jty.nrsc.infrastructure.support.BusinessException;
import com.jty.nrsc.infrastructure.support.ResultCode;
import com.jty.nrsc.interfaces.dto.AuthUserHistoryDto;
import com.jty.nrsc.interfaces.dto.OperatorDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.stereotype.Service;

/**
 * AuthUserServerImpl
 *
 * @author Manjiajie
 * @since 2019-6-12 15:34:47
 */
@Service
public class AuthUserServerImpl implements AuthUserServer {

    private final PasswordEncoder passwordEncoder;
    private final TokenStore tokenStore;

    @Autowired
    public AuthUserServerImpl(PasswordEncoder passwordEncoder, TokenStore tokenStore) {
        this.passwordEncoder = passwordEncoder;
        this.tokenStore = tokenStore;
    }

    @Override
    public String resetPassword(Integer userId) {
        String password = "123456";
        AuthUser user = this.findById(userId);
        user.setPassword(passwordEncoder.encode(password));
        this.removeToken(user);
        user.save();
        return password;
    }

    @Override
    public void enableUser(Integer managerId, OperatorDto operatorDto) {
        DomainEventPublisher.instance().reset();
        DomainEventPublisher.instance().subscriber(new EnableUserSubscriber());
        AuthUser user = this.findById(managerId);
        user.setOperatorDto(operatorDto);
        user.enableUser();
        user.save();
    }

    @Override
    public void disableUser(Integer managerId, OperatorDto operatorDto) {
        DomainEventPublisher.instance().reset();
        DomainEventPublisher.instance().subscriber(new DisableUserSubscriber());
        AuthUser user = this.findById(managerId);
        user.setOperatorDto(operatorDto);
        user.disableUser();
        this.removeToken(user);
        user.save();
    }

    @Override
    public void changePassword(String newPassword, Integer userId) {
        AuthUser authUser = DomainRegistry.userRepository().findById(userId).orElseThrow(()->new BusinessException(ResultCode.USER_NOT_EXIST));
        authUser.setPassword(passwordEncoder.encode(newPassword));
        authUser.save();
    }

    @Override
    public Page<AuthUserHistory> findHistoryByManagerId(AuthUserHistoryDto userHistoryDto){

        AuthUserHistory authUserHistory= new AuthUserHistory();
        authUserHistory.setManagerId(new ManagerId(userHistoryDto.getManagerId()));
        userHistoryDto.setOrderBy("modifiedTime");
        userHistoryDto.setDirection("desc");
        return  DomainRegistry.userHistoryRepository().findAll( Example.of(authUserHistory),userHistoryDto.getPageable());
    }

    private AuthUser findById(Integer id){
        return DomainRegistry.userRepository().findById(id).orElseThrow(()->new BusinessException(ResultCode.USER_NOT_EXIST));
    }

    private void removeToken(AuthUser user){
        tokenStore.findTokensByClientIdAndUserName("*",user.getMobile()).forEach(tokenStore::removeAccessToken );
    }

}
