package com.jty.wsxt.application.service.impl;

import com.jty.wsxt.application.ApplicationRegistry;
import com.jty.wsxt.application.service.NrscTeacherService;
import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.auth.role.AuthRole;
import com.jty.wsxt.domain.model.auth.user.AuthUser;
import com.jty.wsxt.domain.model.auth.user.nrsc.NrscTeacher;
import com.jty.wsxt.domain.model.stage.BasicStage;
import com.jty.wsxt.domain.model.subject.BasicSubject;
import com.jty.nrsc.infrastructure.support.BusinessException;
import com.jty.nrsc.infrastructure.support.ResultCode;
import com.jty.nrsc.interfaces.dto.NrscTeacherDto;
import com.jty.nrsc.interfaces.dto.NrscTeacherSearchDto;
import com.jty.nrsc.interfaces.dto.OperatorDto;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.hibernate.query.criteria.internal.OrderImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.*;
import java.util.ArrayList;
import java.util.List;

/**
 * UserServiceImpl
 *
 * @author Manjiajie
 * @since 2019-6-12 09:58:46
 */
@Service
public class NrscTeacherServiceImpl implements NrscTeacherService {

    private final PasswordEncoder passwordEncoder;

    @Autowired
    public NrscTeacherServiceImpl(PasswordEncoder passwordEncoder) {
        this.passwordEncoder = passwordEncoder;
    }

    @Override
    public Page<NrscTeacher> getNrscTeachers(NrscTeacherSearchDto searchDto) {
        return DomainRegistry.nrscTeacherRepository().findAll(NrscTeacherDsl.getNrscTeachersWhereClause(searchDto),searchDto.getPageable());
    }

    @Override
    public void saveNrscTeacher(NrscTeacherDto nrscTeacherDto) {
        NrscTeacher nrscTeacher;

        AuthUser user = DomainRegistry.userRepository().findByAccount(nrscTeacherDto.getAccount()).orElse(null);
        if(user != null) {
            nrscTeacher =  DomainRegistry.nrscTeacherRepository().findById(user.getId()).orElse(null);
            if(nrscTeacher != null) {
                throw new BusinessException(ResultCode.USER_HAS_EXISTED);
            }
        }
        else {
            user = new AuthUser();
            user.setAccount(nrscTeacherDto.getAccount());
            user.setEnabled(true);
            user.setPassword(passwordEncoder.encode("123456"));
            if(CollectionUtils.isNotEmpty(nrscTeacherDto.getRoleIds())) {
                user.addNrscRolesByIds(nrscTeacherDto.getRoleIds());
            }
        }
        user.setRealName(nrscTeacherDto.getUsername());
        user.save();

        nrscTeacher = new NrscTeacher();
        nrscTeacher.setId(user.getId());
        nrscTeacher.setAuthUser(user);
        nrscTeacher.updateStageId(nrscTeacherDto.getStageId());
        nrscTeacher.updateSubjectId(nrscTeacherDto.getSubjectId());
        nrscTeacher.save();
    }

    @Override
    public void updateNrscTeacher(NrscTeacherDto nrscTeacherDto) {
        NrscTeacher nrscTeacher = DomainRegistry.nrscTeacherRepository().findById(nrscTeacherDto.getId()).orElseThrow(()->new BusinessException(ResultCode.USER_NOT_EXIST));
        AuthUser authUser=nrscTeacher.getAuthUser();
        if(StringUtils.isNotEmpty(nrscTeacherDto.getUsername())){
            authUser.setRealName(nrscTeacherDto.getUsername());
        }
        if(nrscTeacherDto.getSubjectId()!=null){
            BasicSubject subject = new BasicSubject();
            subject.setId(nrscTeacherDto.getSubjectId());
            nrscTeacher.setSubject(subject);
        }
        if(nrscTeacherDto.getStageId()!=null){
            BasicStage stage = new BasicStage();
            stage.setId(nrscTeacherDto.getStageId());
            nrscTeacher.setStage(stage);
        }
        AuthRole.cleanRole(authUser);
        if(CollectionUtils.isNotEmpty(nrscTeacherDto.getRoleIds())) {
            authUser.addNrscRolesByIds(nrscTeacherDto.getRoleIds());
        }
        nrscTeacher.save();
    }

    @Override
    public void changePassword(String newPassword,Integer userId) {
         ApplicationRegistry.authUserServer().changePassword(newPassword,userId);

    }

    @Override
    public String resetPassword(Integer userId) {
        return ApplicationRegistry.authUserServer().resetPassword(userId);
    }

    @Override
    public void enableNrscTeacher(Integer teacherId, OperatorDto operatorDto) {
        ApplicationRegistry.authUserServer().enableUser(teacherId,operatorDto);
    }

    @Override
    public void disableNrscTeacher(Integer teacherId, OperatorDto operatorDto) {
        ApplicationRegistry.authUserServer().disableUser(teacherId,operatorDto);
    }
}
class NrscTeacherDsl {
    public static Specification<NrscTeacher> getNrscTeachersWhereClause(NrscTeacherSearchDto searchDto) {
        return new Specification<NrscTeacher>() {
            @Override
            public Predicate toPredicate(Root<NrscTeacher> root, CriteriaQuery<?> query,
                                         CriteriaBuilder cb) {
                List<Predicate> predicate = new ArrayList<>();
                Join<NrscTeacher, BasicStage> stageJoin = root.join("stage", JoinType.INNER);
                if(searchDto.getStageId() != null) {
                    predicate.add(cb.equal(stageJoin.get("id"), searchDto.getStageId()));
                }

                Join<NrscTeacher, BasicSubject> subjectJoin = root.join("subject", JoinType.INNER);
                if(searchDto.getSubjectId() != null) {
                    predicate.add(cb.equal(subjectJoin.get("id"), searchDto.getSubjectId()));
                }

                Join<NrscTeacher, AuthUser> userJoin = root.join("authUser", JoinType.INNER);
                if(searchDto.getEnabled() != null) {
                    predicate.add(cb.equal(userJoin.get("enabled"), searchDto.getEnabled()));
                }
                if(StringUtils.isNotEmpty(searchDto.getName())) {
                    predicate.add(cb.like(userJoin.get("realName"), "%" + searchDto.getName() + "%"));
                }

                Join<AuthUser, AuthRole> roleJoin = userJoin.join("roles", JoinType.INNER);
                if(searchDto.getRoleId() != null) {
                    predicate.add(cb.equal(roleJoin.get("id"), searchDto.getRoleId()));
                }

                List<Order> orders = new ArrayList<>();
                orders.add(new OrderImpl(root.get("lastLoginTime")));
                orders.add(new OrderImpl(root.get("id")));
                Predicate[] pre = new Predicate[predicate.size()];
                return query.distinct(true).where(predicate.toArray(pre)).orderBy(orders).getRestriction();
            }
        };
    }
}



