package com.jty.wsxt.domain.model.auth.role;

import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.auth.user.AuthUser;
import com.jty.wsxt.domain.model.auth.authority.AuthAuthority;
import com.jty.wsxt.domain.shared.IdentifiedEntityObject;
import com.jty.nrsc.infrastructure.support.BusinessException;
import com.jty.nrsc.infrastructure.support.ResultCode;
import lombok.Data;
import org.apache.commons.lang.StringUtils;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * 用户角色
 *
 * @author Manjiajie
 * @since 2019-6-12 11:01:42
 */
@Entity
@Data
public class AuthRole extends IdentifiedEntityObject<AuthRole> {

    private String name;

    private Integer parentId=0;

    private Integer depth=1;

    private Integer ordinal=1;

    private String  pathIds="";

    private String platform="";

    private String dtype="";

    @ManyToMany(fetch = FetchType.LAZY, mappedBy = "roles")
    private List<AuthUser> users;

    @ManyToMany(cascade = CascadeType.PERSIST, fetch = FetchType.LAZY)
    @JoinTable(name = "auth_relation_role_authority",
            joinColumns = {@JoinColumn(name = "role_id")},
            inverseJoinColumns = {@JoinColumn(name = "authority_id")})
    private List<AuthAuthority> authorities;

    public void addAuthority(AuthAuthority authority){
        if(authorities==null){
            authorities=new ArrayList<>();
        }
        authorities.add( authority );
    }

    public void addAuthoritiesByIds( List<Integer> authAuthoritiesIds){
        authAuthoritiesIds.forEach(authAuthorityId->{
            AuthAuthority authAuthority = DomainRegistry.authAuthorityRepository().findById(authAuthorityId).orElseThrow(()->new BusinessException(ResultCode.DATA_IS_WRONG));
            this.addAuthority(authAuthority);
        });
    }

    @Override
    public boolean sameIdentityAs(AuthRole other) {
        return false;
    }

    public void save(){
        DomainRegistry.roleRepository().save(this);
    }
    public static void cleanRole(AuthUser authUser){
        Iterator<AuthRole> it = authUser.getRoles().iterator();
        while(it.hasNext()) {
            AuthRole role = it.next();
            if(StringUtils.equalsIgnoreCase(role.getPlatform(), "nrsc")) {
                it.remove();
            }
        }
    }
}
