package com.jty.wsxt.domain.model.auth.user;

import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.auth.role.AuthRole;
import com.jty.wsxt.domain.model.auth.history.ManagerId;
import com.jty.wsxt.domain.shared.IdentifiedEntityObject;
import com.jty.nrsc.infrastructure.DomainEventPublisher;
import com.jty.nrsc.infrastructure.support.BusinessException;
import com.jty.nrsc.infrastructure.support.ResultCode;
import com.jty.nrsc.interfaces.dto.OperatorDto;
import lombok.Data;
import org.apache.commons.lang.StringUtils;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * 权限用户
 *
 * @author Manjiajie
 * @since 2019-6-12 11:01:16
 */
@Entity
@Data
@Inheritance(strategy = InheritanceType.JOINED)
public class AuthUser extends IdentifiedEntityObject<AuthUser> {

    @Transient
    private OperatorDto operatorDto;

    private String account;

    private String mobile;

    private String password;

    private Boolean enabled=true;

    private String realName;

    @ManyToMany(cascade = CascadeType.PERSIST, fetch = FetchType.LAZY)
    @JoinTable(name = "auth_relation_user_role",
            joinColumns = {@JoinColumn(name = "user_id")},
            inverseJoinColumns = {@JoinColumn(name = "role_id")})
    private List<AuthRole> roles=new ArrayList<>();

    @Override
    public boolean sameIdentityAs(AuthUser other) {
        return false;
    }

    public void addNrscRoles(List<AuthRole> roleList){
        Iterator<AuthRole> it = this.getRoles().iterator();
        while(it.hasNext()){
            AuthRole role=it.next();
            if(StringUtils.equalsIgnoreCase(role.getPlatform(),"nrsc")){
                it.remove();
            }
        }
        roleList.forEach(role -> {
            if(role.getPlatform().equals("nrsc")){
                roles.add(role);
            }
        });
    }

    public void addNrscRolesByIds( List<Integer> roleIds){
        List<AuthRole> roleList = new ArrayList<>();
        roleIds.forEach(roleId -> {
            roleList.add(DomainRegistry.roleRepository().findById(roleId).orElseThrow(() -> new BusinessException(ResultCode.DATA_IS_WRONG)));
        });
        this.addNrscRoles(roleList);
    }

    public void enableUser(){
        this.setEnabled(true);
        DomainEventPublisher.instance().publish(new EnableUserEvent(operatorDto,new ManagerId(this.getId())));
    }

    public void disableUser(){
        this.setEnabled(false);
        DomainEventPublisher.instance().publish(new DisableUserEvent(operatorDto,new ManagerId(this.getId())));
    }

    public void save(){
        DomainRegistry.userRepository().save(this);
    }

}
