package com.jty.wsxt.infrastructure.security;

import com.jty.wsxt.domain.model.auth.client.AuthClient;
import com.jty.wsxt.domain.model.auth.client.ClientRepository;
import com.jty.wsxt.infrastructure.support.BusinessException;
import com.jty.wsxt.infrastructure.support.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.ClientDetails;
import org.springframework.security.oauth2.provider.ClientDetailsService;
import org.springframework.security.oauth2.provider.ClientRegistrationException;
import org.springframework.security.oauth2.provider.client.BaseClientDetails;
import org.springframework.stereotype.Service;

import java.util.Arrays;

/**
 * 资源服务器的自定义Client配置获取
 *
 * @author Jason
 * @since 2018/12/14 08:51
 */
@Service
public class MyClientDetailService implements ClientDetailsService {

    private final ClientRepository clientRepository;

    @Autowired
    public MyClientDetailService(ClientRepository clientRepository) {
        this.clientRepository = clientRepository;
    }

    @Override
    public ClientDetails loadClientByClientId(String clientId) throws ClientRegistrationException {
        AuthClient client=clientRepository.findByClientId(clientId).orElseThrow(()->new BusinessException(ResultCode.USER_NOT_EXIST));
        BaseClientDetails clientDetails=new BaseClientDetails();
        clientDetails.setClientId(client.getClientId());
        clientDetails.setClientSecret(client.getClientSecret());
        clientDetails.setAuthorizedGrantTypes(Arrays.asList( client.getAuthorizedGrantTypes().split(",")));
        clientDetails.setScope(Arrays.asList( client.getScope().split(",")));
        return clientDetails;
    }
}
